package com.kingdee.shr.perfweb.portal.handler;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.util.app.DbUtil;
import com.kingdee.jdbc.rowset.IRowSet;
import com.kingdee.shr.base.syssetting.context.SHRContext;
import com.kingdee.shr.base.syssetting.exception.SHRWebException;
import com.kingdee.shr.base.syssetting.web.json.JSONUtils;
import com.kingdee.shr.perfweb.app.base.evalplan.*;
import com.kingdee.shr.perfweb.app.base.evalrule.WFNodeDefInfo;
import com.kingdee.shr.perfweb.app.common.YesOrNoEnum;
import com.kingdee.shr.perfweb.app.exception.SHRPerfWebBizException;
import com.kingdee.shr.perfweb.app.resource.SHRPerfWebResource;
import com.kingdee.shr.perfweb.app.workflow.*;
import com.kingdee.shr.perfweb.bean.PerfScoreBatchSaveBean;
import com.kingdee.shr.perfweb.bean.PerfScoreList;
import com.kingdee.shr.perfweb.exception.SHRPerfWebException;
import com.kingdee.shr.perfweb.portal.handler.PerfPersonalScoreHandler;
import com.kingdee.shr.perfweb.util.EvaFormUtil;
import com.kingdee.shr.perfweb.util.GetEntityInfoUtils;
import com.kingdee.shr.perfweb.util.PerfPortalUtils;
import com.kingdee.shr.perfweb.util.SHRPerfWebResourceUtil;
import com.kingdee.shr.perfweb.util.ServerResponseConst;
import com.kingdee.shr.perfweb.utils.RequestUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.ui.ModelMap;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.SQLException;
import java.util.*;


/**
 * 绩效评分拓展
 * @author coyle
 * 20241213
 *
 */
public class PerfPersonalScoreHandlerEx extends PerfPersonalScoreHandler {
    private Logger logger = LoggerFactory.getLogger(PerfPersonalScoreHandlerEx.class);
    


    public void saveScoreAction(HttpServletRequest request, HttpServletResponse response, ModelMap modelMap) throws SHRWebException, BOSException, EASBizException {
        Context ctx = SHRContext.getInstance().getContext();
        Map<String, Object> res = new HashMap();
        String scoreObj = RequestUtils.getParameter(request, "scoreObj");
        String isRealSave = RequestUtils.getParameter(request, "isRealSave");
        Gson gson = new Gson();
        List<PerfScoreList> perfScoreListAll = (List)gson.fromJson(scoreObj, (new TypeToken<List<PerfScoreList>>() {
        }).getType());
        if (CollectionUtils.isEmpty(perfScoreListAll)) {
            JSONUtils.writeJson(response, res);
        } else { 
            PerfScoreList perfScoreList = perfScoreListAll.get(0);
            String sumScore = perfScoreList.getSumScore();
            String evaObj = perfScoreList.getEvaObj();
            PerfEvaObjectInfo info = GetEntityInfoUtils.getPerfEvaObjectInfo(ctx, evaObj);
            Set<String> nodeTask = new HashSet();
            Map<String, String> nodeScore = new HashMap();
            List<PerfScoreList> normalScoreList = new ArrayList();
            List<PerfScoreBatchSaveBean> sumScoreList = new ArrayList();
            Iterator var16 = perfScoreListAll.iterator();

            while(var16.hasNext()) {
                PerfScoreList pl = (PerfScoreList)var16.next();
                PerfScoreBatchSaveBean bean = new PerfScoreBatchSaveBean();
                bean.setGradeTaskNodeInstID(pl.getGradeTaskNode());
                bean.setWholeComment(pl.getWholeComment());
                bean.setNodeGrade(pl.getWholeGrade());
                bean.setNodeSumScore(pl.getSumScore());
                bean.setEvaObj(pl.getEvaObj());
                if ("3".equals(pl.getNeedScore())) {
                    sumScoreList.add(bean);
                } else {
                    normalScoreList.add(pl);
                }
            }

            String saveStatus;
            Map map;
            if (sumScoreList.size() > 0) {
                IPerfWorkFlowFacade iPerfWorkFlowFacade = PerfWorkFlowFacadeFactory.getLocalInstance(ctx);
                map = iPerfWorkFlowFacade.saveNodeSumScoreAndGrade(sumScoreList);
                String msg = SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.SAVESUCCESS, ctx);
                if (map.get("code").toString().equalsIgnoreCase(ServerResponseConst.SUCCESS)) {
                    res.put("result", "200");
                } else {
                    res.put("result", "500");
                    saveStatus = map.get("code").toString();
                    if (ServerResponseConst.FAILED.equals(saveStatus)) {
                        msg = SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.ALLHASSCOREFORMULAR, ctx);
                    } else if ("400".equals(saveStatus)) {
                        msg = SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.OVERSUMRANGE, ctx);
                    } else if ("999".equals(saveStatus)) {
                        msg = SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.LESSTHANQUANTITYS, ctx);
                    } else if ("LESSLOWLIMIT".equals(saveStatus)) {
                        msg = (new SHRPerfWebBizException(SHRPerfWebBizException.SCOREOVERMIN2)).getMessage();
                    } else if (ServerResponseConst.INVALID.equals(saveStatus)) {
                        msg = SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.CHANGEHANDLER, ctx);
                    }
                }

                res.put("msg", msg);
                res.put("maxCanPut", map.get("maxCanPut"));
                res.put("gradetaskNode", sumScoreList);

                res.put("nodeGrade", ((PerfScoreBatchSaveBean)sumScoreList.get(0)).getNodeGrade());
                res.put("detail", "");
                if ("500".equals(res.get("result"))) {
                    JSONUtils.writeJson(response, res);
                    return;
                }
            }

            if (normalScoreList.size() > 0) {
//            	PerfWorkFlowFacadeControllerBeanEx.getInstance().calculateAndSaveScore(ctx, normalScoreList, res, isRealSave, nodeTask, nodeScore);
            }

            StringBuffer msgInfo = new StringBuffer("");
            if (info.getCanAdjustEvaluer().getValue() == 1) {
                try {
                    map = getAreaBaseNameByNodeTaskInst(nodeTask, evaObj);
                    Iterator<String> it = nodeScore.keySet().iterator();

                    while(it.hasNext()) {
                        saveStatus = (String)it.next();
                        String[] grade = ((String)nodeScore.get(saveStatus)).split("#");
                        String areaName = (String)map.get(saveStatus);
                        GradeTaskNodeInstInfo gtnInfo = GetEntityInfoUtils.getGradeTaskNodeInstInfoById(ctx, saveStatus);
                        WFNodeDefInfo wfNodeInfo = GetEntityInfoUtils.getWFNodeDefInfo(ctx, gtnInfo.getWfNode().getId().toString());
                        if (grade.length > 1) {
                            msgInfo.append(areaName);
                            if (!wfNodeInfo.getNeedScore().getValue().equals("0")) {
                                msgInfo.append(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.SCOREIS, ctx)).append(":").append(grade[0]).append(",");
                            }

                            if (wfNodeInfo.getPerfGradeEvaluate().getValue().equals("1")) {
                                msgInfo.append(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.GRADEIS, ctx)).append(":").append(grade[1]).append("，");
                            }
                        } else {
                            String gradeStr = "";
                            if (grade.length > 0) {
                                gradeStr = grade[0];
                            }

                            msgInfo.append(areaName).append(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.SCOREIS, ctx)).append(":").append(gradeStr);
                            if (wfNodeInfo.getPerfGradeEvaluate().getValue().equals("1")) {
                                msgInfo.append(",").append(SHRPerfWebResourceUtil.getRes(SHRPerfWebResource.GRADEIS, ctx)).append(":");
                            }
                        }
                    }
                } catch (BOSException var25) {
                    this.logger.error("", var25);
                }
            }
            //sumScore 20241214   ((PerfScoreBatchSaveBean)sumScoreList.get(0)).getNodeSumScore() 改sumScore
            res.put("nodeSumScore", sumScore);
            res.put("detail", msgInfo.toString());
            JSONUtils.writeJson(response, res);
        }
    }



    private Map<String, String> getAreaBaseNameByNodeTaskInst(Set nodeTakSet, String evaObj) throws BOSException {
        Context ctx = SHRContext.getInstance().getContext();
        Map<String, String> map = new HashMap();
        GradeTargetInstCollection coll = GetEntityInfoUtils.getGradeTargetInstCollectionByNodeInst(ctx, nodeTakSet, evaObj);
        Set<String> exist = new HashSet();

        for(int i = 0; i < coll.size(); ++i) {
            if (map.get(coll.get(i).getGradeTaskNode().getId().toString()) != null) {
                if (!exist.contains(coll.get(i).getAreaBase().getId().toString()) && coll.get(i).getEvaObj().getCanAdjustEvaluer().getValue() != 0) {
                    String areaName = (String)map.get(coll.get(i).getGradeTaskNode().getId().toString());
                    areaName = areaName + "," + coll.get(i).getAreaBase().getName();
                    map.put(coll.get(i).getGradeTaskNode().getId().toString(), areaName);
                    exist.add(coll.get(i).getAreaBase().getId().toString());
                }
            } else if (coll.get(i).getEvaObj().getCanAdjustEvaluer().getValue() == 0) {
                map.put(coll.get(i).getGradeTaskNode().getId().toString(), "");
            } else {
                map.put(coll.get(i).getGradeTaskNode().getId().toString(), coll.get(i).getAreaBase().getName());
                exist.add(coll.get(i).getAreaBase().getId().toString());
            }
        }

        GradeTaskNodeInstCollection nodecoll = GetEntityInfoUtils.getGradeTaskNodeInstCollByEvaObj(ctx, evaObj);
        PerfEvaObjectInfo evaInfo = GetEntityInfoUtils.getPerfEvaObjectInfo(ctx, evaObj);
        PerfEvaObjMatrixWorkFlowInstCollection matriInfotColl = PerfEvaObjMatrixWorkFlowInstFactory.getLocalInstance(ctx).getPerfEvaObjMatrixWorkFlowInstCollection("select *, areaBase.name where evaObj ='" + evaObj + "'");
        PerfPeriodInfo perfPeriod = evaInfo.getPeriod();
        String perfPlanId = perfPeriod.getPerfPlan().getId().toString();
        String perfRuleId = perfPeriod.getPerfRule().getId().toString();
        YesOrNoEnum isHistoryUpdate = perfPeriod.getIsHistoryUpdate();
        PerfEvaObjectMatrixWorkFlowCollection matriWorkFlow = null;
        matriWorkFlow = PerfEvaObjectMatrixWorkFlowFactory.getLocalInstance(ctx).getPerfEvaObjectMatrixWorkFlowCollection("select *, areaBase.name where perfplan = '" + perfPlanId + "' and perfRule = '" + perfRuleId + "'");
        if (matriWorkFlow.isEmpty() && isHistoryUpdate.getValue() == 1) {
            matriWorkFlow = PerfEvaObjectMatrixWorkFlowFactory.getLocalInstance(ctx).getPerfEvaObjectMatrixWorkFlowCollection("select *, areaBase.name where perfplan = '" + perfPlanId + "' and perfRule is null");
        }

        Map<String, String> worfFlowAreaMap = new HashMap();

        int i;
        for(i = 0; i < matriInfotColl.size(); ++i) {
            PerfEvaObjMatrixWorkFlowInstInfo matriInfo = matriInfotColl.get(i);
            if (StringUtils.isEmpty((CharSequence)worfFlowAreaMap.get(matriInfo.getWorkflow().getId().toString()))) {
                worfFlowAreaMap.put(matriInfo.getWorkflow().getId().toString(), matriInfo.getAreaBase().getName());
            }
        }

        for(i = 0; i < matriWorkFlow.size(); ++i) {
            PerfEvaObjectMatrixWorkFlowInfo matriWorkFlowInfo = matriWorkFlow.get(i);
            if (StringUtils.isEmpty((CharSequence)worfFlowAreaMap.get(matriWorkFlowInfo.getWorkflow().getId().toString()))) {
                worfFlowAreaMap.put(matriWorkFlowInfo.getWorkflow().getId().toString(), matriWorkFlowInfo.getAreaBase().getName());
            }
        }

        for(i = 0; i < nodecoll.size(); ++i) {
            GradeTaskNodeInstInfo info = nodecoll.get(i);
            if (StringUtils.isEmpty((CharSequence)map.get(info.getId().toString()))) {
                String workFlowId = info.getWfNode().getWorkflow().getId().toString();
                map.put(info.getId().toString(), worfFlowAreaMap.get(workFlowId));
            }
        }

        return map;
    }



    public void getDefaultScoreAction(HttpServletRequest request, HttpServletResponse response, ModelMap modelMap) throws SHRWebException {
        Context ctx = SHRContext.getInstance().getContext();
        Map<String, Object> res = new HashMap();
        try {
            String evaObjVal = RequestUtils.getParameter(request, "evaObjVal");
            String levelName =  com.kingdee.util.StringUtils.cnulls(RequestUtils.getParameter(request, "levelName")); //
            if (StringUtils.isBlank(levelName) || StringUtils.isBlank(evaObjVal) ) {
                res.put("default", 0);
                JSONUtils.writeJson(response, res);
                return;
            }

            
            StringBuilder sbu = new StringBuilder();
            sbu.append(" /*dialect*/ select define.FDefaultScore deScore from T_EVA_PerfEvaObject evaobj ");
            sbu.append(" left join T_EVA_PWorkFlowDef def on evaobj.FWorkFlowID = def.fid ");
            sbu.append(" left join T_EVA_PerformanceRule erule on def.FPerfRule = erule.fid ");
            sbu.append(" left join T_PER_PerfScoreLevel slevel on erule.FPerfRateID = slevel.fid ");
            sbu.append(" left join T_PER_PerfScorLevelDefine define on define.FPerfScoreLevelID = slevel.fid ");
            sbu.append(" where define.FPerfLevelName = ? and evaobj.fid = ?");
            
            IRowSet rowSet = DbUtil.executeQuery(ctx, sbu.toString(), new Object[]{levelName, evaObjVal});
            
            // 优化：去除小数,只保留整数的逻辑在默认值不为0时才需要
            res.put("default", 0);
            if (rowSet.next()) {
            	double deScore = rowSet.getDouble("deScore");
            	res.put("default", Math.round(deScore * 100.0) / 100.0);
            }
            
            JSONUtils.writeJson(response, res);
              

        } catch (IllegalArgumentException e) {
            logger.error("Invalid argument error", e);
            res.put("error", e.getMessage());
            res.put("result", "400");
            JSONUtils.writeJson(response, res);
        } catch (SQLException e) {
            logger.error("SQL execution error", e);
            res.put("error", "数据库查询执行失败");
            res.put("result", "500");
            JSONUtils.writeJson(response, res);
        } catch (Exception e) { // 捕获其他可能的异常
            logger.error("Unexpected error", e);
            res.put("error", "发生未知错误");
            res.put("result", "500");
            JSONUtils.writeJson(response, res);
        }
    }

    public void checkGradeHasAction(HttpServletRequest request, HttpServletResponse response, ModelMap modelMap) throws  SHRWebException {
        Context ctx = SHRContext.getInstance().getContext();
        Map<String, Object> res = new HashMap();
        try {
            String evaObjVal = RequestUtils.getParameter(request, "evaObjVal");
            String scoreVal = com.kingdee.util.StringUtils.cnulls(RequestUtils.getParameter(request, "scoreVal")); //  
            String levelName = com.kingdee.util.StringUtils.cnulls(RequestUtils.getParameter(request, "levelName")); 
            if (StringUtils.isBlank(scoreVal) || StringUtils.isBlank(evaObjVal)) {
                res.put("has", 0);
            }

            StringBuilder sbu = new StringBuilder();
            sbu.append("  /*dialect*/  select  count( define.FPerfLevelName) countNum   from T_EVA_PerfEvaObject evaobj ");
            sbu.append(" left join T_EVA_PWorkFlowDef def on evaobj.FWorkFlowID = def.fid ");
            sbu.append(" left join T_EVA_PerformanceRule erule on def.FPerfRule = erule.fid ");
            sbu.append(" left join T_PER_PerfScoreLevel slevel on erule.FPerfRateID = slevel.fid ");
            sbu.append(" left join T_PER_PerfScorLevelDefine define on define.FPerfScoreLevelID = slevel.fid ");
            sbu.append(" where define.FScoreLowLimit <= ? and   define.FScoreUpLimit >= ? and evaobj.fid = ? ");
            sbu.append(" and define.FPerfLevelName = ?  ");
            
            IRowSet rowSet = DbUtil.executeQuery(ctx, sbu.toString(), new Object[]{scoreVal,scoreVal, evaObjVal,levelName});
            //如果rowSet存在两条则返回1，否则返回0
            while (rowSet.next())    {
            	int countNum = rowSet.getInt("countNum");
            	if (countNum >= 1) {
            		res.put("has", 1);
            	} else {
            		res.put("has", 0);
            	}
            }
            JSONUtils.writeJson(response, res);
      
             

        } catch (IllegalArgumentException e) {
            logger.error("Invalid argument error", e);
            res.put("error", e.getMessage());
            res.put("result", "400");
            JSONUtils.writeJson(response, res);
        } catch (SQLException e) {
            logger.error("SQL execution error", e);
            res.put("error", "数据库查询执行失败");
            res.put("result", "500");
            JSONUtils.writeJson(response, res);
        } catch (Exception e) { // 捕获其他可能的异常
            logger.error("Unexpected error", e);
            res.put("error", "发生未知错误");
            res.put("result", "500");
            JSONUtils.writeJson(response, res);
        }
    }

}
